/*
 *  SMTPproxy.cpp
 *  eProxy
 *
 *  Created by Raymond Doetjes on 6/6/05.
 *  Copyright 2005 Phonax.com. All rights reserved.
 *  email: rdoetjes@phonax.com
 */

#include "SMTPproxy.h"
#include <string>

using namespace std;

SMTPproxy::SMTPproxy(Socket* csock, string id)
{
  sock=csock;  
  sock->Send(string("220 ")+id+string("\r\n"));
}

bool SMTPproxy::checkResult(const char *response)
{
  string tmp(response);
  bool result=false;
  
  if (tmp.find(string("250 "))!=string::npos)
    result=true;
  else
    result=false;
  return result;
}

bool SMTPproxy::CmdCompare(string cmd, string cmp)
{
  transform(cmd.begin(),cmd.end(), cmd.begin(), (int(*)(int)) toupper);
  transform(cmp.begin(),cmp.end(), cmp.begin(), (int(*)(int)) toupper);
  if (cmd.find(cmp)==0)
    return  true;
  else
    return false;
}

string SMTPproxy::getDomainName(string cmd)
{
  string result("");
  result=getEmailAdres(cmd);
  int bpos=0;
  bpos=result.find("@", 0);
  result=result.substr(bpos+1, result.length());
  return result;
}


string SMTPproxy::getEmailAdres(string cmd)
{
  string result("");
  int epos, bpos=0;
  bpos=cmd.find("<",0);
  epos=cmd.find(">",bpos);
  result=cmd.substr(bpos+1,(epos-bpos)-1);
  return result;
}

void SMTPproxy::Run(int (*from)(string sMailFrom), int(*rcptto)(string sRcptTo), int (*body)(string sBody), void(*email)(string sEmail), int sec_smtp_port)
{
  char buffer[BUFFSIZE];
  int iStatus=HELO;
  string cmd("");
  string sEmail("");
  string sBody("");
  bzero(buffer, BUFFSIZE);
  int iBodyStat;

  Socket smtpC(0, "127.0.0.1", sec_smtp_port);
  smtpC.Connect();
  smtpC.Read(buffer, BUFFSIZE-1);
  
  do 
  {
	bzero(buffer, BUFFSIZE);
	cmd="";
	
    if (sock->ReadLine(buffer,BUFFSIZE-1)==-1) //socket error so disconnect
	  return;
	  
	cmd=buffer;
	sEmail+=cmd;
	bzero(buffer, BUFFSIZE);
	
	if (CmdCompare(cmd, string("XFORWARD"))) //XFORWARD can occur anywhere betweeh EHLO and DATA
	{
	  smtpC.Send(cmd);
	  smtpC.Read(buffer, BUFFSIZE-1);
	  sock->Send(buffer, strlen(buffer));
	}
	
	if (iStatus==HELO && (CmdCompare(cmd, string("HELO ")) || CmdCompare(cmd, string("EHLO "))))
	{
	  smtpC.Send(cmd);
	  smtpC.Read(buffer, BUFFSIZE-1);
	  sock->Send(buffer, strlen(buffer));	    
	  if (checkResult(buffer))
	    iStatus=MAILFROM;
	}

	if (iStatus==MAILFROM && CmdCompare(cmd, string("MAIL FROM:")))
	{
	  smtpC.Send(cmd);
	  smtpC.ReadLine(buffer, BUFFSIZE-1);
	  if ( from(getEmailAdres(cmd)) < 0 )
	  {
		sock->Send(string("550 connection delibrately broken off at MAIL FROM stage\r\n"));
		iStatus=BOUNCE;
	  }
	  else
	  {
	    sock->Send(buffer, strlen(buffer));  
		if (checkResult(buffer))
	      iStatus=RCPTTO;
	  }
	}

	if ( (iStatus==RCPTTO || iStatus==DATA) && CmdCompare(cmd, string("RCPT TO:"))) // One can have several rcpt to requests!
	{
	  smtpC.Send(cmd);
	  smtpC.ReadLine(buffer, BUFFSIZE-1);
	  if ( rcptto(getEmailAdres(cmd)) < 0 )
	  {
		sock->Send(string("550 connection delibrately broken off at RCPTO stage\r\n"));
		iStatus=BOUNCE;
	  }
	  else 
	  {
		sock->Send(buffer, strlen(buffer));
		if (checkResult(buffer))
	      iStatus=DATA;
	  }
	}
	
	if (iStatus!=DOT && CmdCompare(cmd, string("QUIT\r\n")))
	{
	  smtpC.Send(cmd);
	  iStatus=BOUNCE;
	}
	
	//Read the body and add it to the sBody string, this if clause needs to be infront of the 354 End data if clause. To prevent a double DATA line
	if (iStatus==DOT && cmd!=".\r\n")
	{
	  sBody+=cmd;
	}
	else if (iStatus==DOT && cmd==".\r\n")
	{
	  iStatus=DONE;
	}
	
	if (iStatus==DATA && CmdCompare(cmd, string("DATA\r\n")))
	{
	  sock->Send(string("354 End data with <CR><LF>.<CR><LF>\r\n"));  
	  iStatus=DOT;
	}
  }while( iStatus!=DONE && iStatus!=BOUNCE );
  
  if (iStatus==DONE)
  {
	iBodyStat=body(sBody);
	if (iBodyStat>0)
	{
      smtpC.Send(string("DATA\r\n"));
	  bzero(buffer, BUFFSIZE);
      smtpC.Read(buffer, BUFFSIZE-1);
	  smtpC.Send(sBody);
	  smtpC.Send(string(".\r\n"));
      bzero(buffer, BUFFSIZE);
      smtpC.Read(buffer, BUFFSIZE-1);
	  sock->Send(buffer, strlen(buffer));
     }
     else if (iBodyStat<0)
     {
       sock->Send(string("550 The content of this email is reason to disallow it for delivery!\r\n"));
	 }
  }	
  
  smtpC.Close();
  email(sEmail);
}

SMTPproxy::~SMTPproxy()
{

}
