#ifndef H_SOCKET
#include "Socket.h"
#define H_SOCKET
#endif
#include <string.h>
#include <math.h>
#include <errno.h>

Socket::Socket(int fd, char *host, int port, string sRemark)
{ 
  Remark=sRemark;
  act.sa_handler=SIG_IGN;
  sigemptyset(&act.sa_mask);
  act.sa_flags=0;
  reconnect=0;

  if (sigaction(SIGPIPE, &act, NULL) <0)
  {
    perror("Could not catch SIGPIPE");
  }

  if (fd!=0)
    setSocket(fd);
  else
  {
    hp=gethostbyname(host);
    if (!hp)
      hp=gethostbyaddr(host, strlen(host), AF_INET);
    if (!hp)
    {
      printf("Can't resolve server name/address\r\n");
      exit(1);
    }
    s.sin_addr = *(struct in_addr *) hp->h_addr_list[0];
    s.sin_family = hp->h_addrtype;
    s.sin_port = htons(port);
    setSocket(socket(AF_INET, SOCK_STREAM, 0));
  }

  //Reset the fdset fdr en dfw to zero
  //these 'structures' are used to hold socket status 
  FD_ZERO(&fdr);
  FD_SET(fd, &fdr);
  FD_ZERO(&fdw);
  FD_SET(fd, &fdw);
}

void Socket::setRemark(string sRemark)
{
  Remark=sRemark;
}

string Socket::getRemark()
{
  return Remark;
}


bool Socket::Connected()
{
  if (recv(sock, NULL, 1, MSG_PEEK)==0)
    isConnected=false;
  return isConnected;
}

int Socket::Reconnect()
{
  int result=-1;
  //this is a global used only from external progranm to see if reconnect was finished.
  result=result;
  close(sock);
  setSocket(socket(AF_INET, SOCK_STREAM, 0));
  nonBlocking();
  return Connect();
}

void Socket::nonBlocking()
{ 
   fcntl(sock, F_SETFL, O_NONBLOCK);
}

int Socket::Connect()
{
  int result=-1;
  if ( connect(getSocket(), (struct sockaddr *)&s, sizeof(s)) == -1 )
    result=-1;
  else
    result=1;
  return result;
}

void Socket::setTimeOut(int timeout=0)
{
  to.tv_sec=timeout;
  to.tv_usec=0;

  setsockopt(sock+1, SOL_SOCKET, SO_RCVTIMEO, &to, sizeof(to));
  setsockopt(sock+1, SOL_SOCKET, SO_SNDTIMEO, &to, sizeof(to));
}

void Socket::setSocket(int fd)
{
  sock=fd;
  isConnected=true;
}

int Socket::getSocket()
{
  return sock;
}

int Socket::fdRdySend(int sec=0, int usec=0)
{
  int rc;
  tv.tv_sec = sec;
  tv.tv_usec = usec;

  rc = select(sock+1, NULL, &fdw, NULL, &tv);
  if (errno==EPIPE)
    isConnected=false;
  //return FD_ISSET(sock+1,&fdw) ? 1 : 0;
  return rc;
}

int Socket::fdRdyRead(int sec=0, int usec=0)
{
  int rc;
  tv.tv_sec = sec;
  tv.tv_usec = usec;

  FD_ZERO(&fdr);
  FD_SET(sock, &fdr);

  rc = select(sock+1, &fdr, NULL, NULL, &tv);
  if (errno==EPIPE)
    isConnected=false;
  //return FD_ISSET(sock+1,&fdr) ? 1 : 0;
  return rc;
}

int Socket::fdRdy(int sec=0, int usec=0)
{
  int rc;
  tv.tv_sec = sec;
  tv.tv_usec = usec;
 
  FD_ZERO(&fdw);
  FD_SET(sock, &fdw);

  rc = select(sock+1, NULL, &fdw, &fdr, &tv);
  if (errno==EPIPE)
    isConnected=false;
  //return FD_ISSET(sock+1,&fdw) ? 1 : 0;
  return rc;
}

void Socket::Close()
{
  shutdown(sock, SHUT_RDWR);
  close(sock); 
}

int Socket::WriteLine(char *data, FILE *rcv)
{
  int result=0;
  if (!feof(rcv) )
  {
    fputs(data, rcv);
    result=1;
  }
  else
    result=0;

  return result;
}

int Socket::ReadLine(char *data, int maxlen=4096)
{
  int result=0;
  int bytes=0;
  int i=0;
  char *buffer=new char[maxlen];
  if (buffer==NULL)
    return -1;
	
  bzero(buffer, maxlen);
  
  bytes=Peek(buffer, maxlen-1);
  if (bytes==-1)
    return bytes;
	
  if(bytes>0)
  {
    do
	{
	  if (buffer[i]=='\n')
	  {
	    Read(data, i+1);
		result=i;
	  }
	  i++;
	}while(i<bytes && result==0);
  }
  delete []buffer, NULL;
  return result;
}

int Socket::Peek(char *data, int len)
{
  int bytes=0;
  bytes=recv(sock, data, len, MSG_PEEK);
  return bytes;
}

int Socket::Read(char *data, int len)
{
  int result=0;
  result=read(sock, data, len);
  if (errno==EPIPE)
    isConnected=false;
  return result;
}

int Socket::Send(string data)
{
  int result=0;
  int len=data.length();
  int length=len;
  char *cdata;

  cdata=new char[len+3];
  if (cdata==NULL)
    return -1;
  bzero(cdata, len);
  memcpy(cdata, data.c_str(), len);

  int totaal=0;
  do
  {
    result=write(sock, cdata, len); 
    if (result!=-1)
    {
      totaal=totaal+result;
      len=len-result;
      memcpy(cdata, cdata+result, len);
    }
    usleep(50);
  } while ( totaal!=length && errno!=EPIPE);
  
  if (errno==EPIPE)
    isConnected=false;

  if (cdata)
    delete []cdata, NULL;
  return length;
}

int Socket::Send(char *data, int len)
{
  int result=0;
  int length=len;
  char *cdata;

  cdata=new char[len+3];
  if (cdata==NULL)
    return -1;
  bzero(cdata, len);
  memcpy(cdata, data, len);

  int totaal=0;
  do
  {
    result=write(sock, cdata, len); 
    if (result!=-1)
    {
      totaal=totaal+result;
      len=len-result;
      memcpy(cdata, cdata+result, len);
    }
    usleep(50);
  } while ( totaal!=length && errno!=EPIPE);
  
  if (errno==EPIPE)
    isConnected=false;

  if (cdata)
    delete []cdata, NULL;
  return length;
}  

int Socket::sendFile(char *file, int buffer)
{
  int sfile;
  char cbuffer[buffer];
  int bytes=0;
  int totalbytes=0;
                                                                                
  sfile=open(file, O_RDONLY);
  if (sfile)
  {
    do {
      bzero(cbuffer, buffer);
      bytes=read(sfile, cbuffer, buffer);
      Send(cbuffer, bytes);
      totalbytes+=bytes;
      usleep(5000);
    } while (bytes != 0);
  }
  else
    return 0;
  close(sfile);
  return totalbytes;
}

Socket::~Socket()
{
  Close();
}
