#include "serverSocket.h"

serverSocket::serverSocket(int port, u_long addr)
{
  srvPort=port;
  srvSocket=-1;
  srvBacklog=0;
  bindAddr=addr;
  ReuseAddr=true;
  ReusePort=true;

  //Reset the fdset fdr en dfw to zero
  //these 'structures' are used to hold socket status 
  FD_ZERO(&fdr);
  FD_ZERO(&fdw);

  signal (SIGPIPE, SIG_IGN);
}

bool serverSocket::fdRdySend(int fd)
{
  int rc;
  tv.tv_sec = tv.tv_usec = 0;

  rc = select(fd+1, NULL, &fdw, NULL, &tv);
  if (rc < 0)
    return -1;

  return FD_ISSET(fd,&fdw) ? 1 : 0;
}

bool serverSocket::fdRdyRead(int fd)
{
  int rc;
  tv.tv_sec = tv.tv_usec = 0;

  rc = select(fd+1, &fdr, NULL, NULL, &tv);
  if (rc < 0)
    return -1;

  return FD_ISSET(fd,&fdr) ? 1 : 0;
}

void serverSocket::nonBlocking(void)
{
   fcntl(srvSocket, F_SETFL, O_NONBLOCK);
}

int serverSocket::startListener(bool nonblocking, bool manual)
{
  /*When manual set to false then Bind and Listen(5) are called automatically without error handling
    when you wish to handle teh errors yourself, please use startListener(true)*/

  srvSocket=socket(PF_INET, SOCK_STREAM, 0);

  //When no free socket could be found discontinue
  if (srvSocket == -1)
    return 0;

  if (nonblocking)
    nonBlocking();

  //Set the sock options
  setsockopt(srvSocket,SOL_SOCKET, SO_REUSEADDR, &ReuseAddr, sizeof(ReuseAddr));
  #ifdef SO_REUSEPORT
  setsockopt(srvSocket,SOL_SOCKET, SO_REUSEPORT, &ReusePort, sizeof(ReusePort));
  #endif

  bzero(&srvAddr, sizeof(srvAddr));
  srvAddr.sin_family=AF_INET;
  srvAddr.sin_addr.s_addr = htonl(bindAddr); 
  srvAddr.sin_port=htons(srvPort);
  if (!manual)
  {
    if ( Bind() !=-1)
    {
      if ( Listen(5) !=-1)
        return 1;
      else 
        return 0;
    }
    return 0;
  }//end of if(!manual)
  else
    return 1;
}

int serverSocket::Bind(void)
{
  if (bind(srvSocket, (struct sockaddr *)&srvAddr, sizeof(srvAddr)) == -1)
    return -1;
  else
    return 1;
}

int serverSocket::Listen(int backlog)
{
  srvBacklog=backlog;
  if ( listen(srvSocket, backlog) ==-1)
    return -1;
  else
    return 1;
}

int serverSocket::Accept(void)
{
  int sock;
  sock=accept(srvSocket, (struct sockaddr *)&clntAddr, &len);
  return sock;
}

void serverSocket::Close(int socket)
{
  shutdown(socket, SHUT_RDWR);
  close(socket); 
}

serverSocket::~serverSocket()
{
  Close(srvSocket);
}
